// Copyright (c) 2003 Charles Samuels <charles@kde.org>
// See the file COPYING for redistribution terms.

#include "tree.h"
#include "selector.h"
#include "oblique.h"

#include <noatun/app.h>
#include <noatun/player.h>

Item::Item(const File &file)
	: mFile(file)
{

}

QString Item::property(const QString &key, const QString &def) const
{
	if (key == "url")
	{
		KURL url;
		url.setPath(property("file"));
		return url.url();
	}

	QString str = mFile.property(key);
	if (str.isNull()) return def;
	return str;
}

void Item::setProperty(const QString &key, const QString &property)
{
	mFile.setProperty(key, property);
}

void Item::clearProperty(const QString &key)
{
	(void)key;
	// TODO
}

QStringList Item::properties() const
{
	return QStringList(); // TODO
}

bool Item::isProperty(const QString &key) const
{
	return !mFile.property(key).isNull();
}

bool Item::operator==(const PlaylistItemData &d) const
{
	return mFile == static_cast<const Item&>(d).mFile;
}

void Item::remove()
{
	// TODO
}



Selector::Selector()
{
}

Selector::~Selector()
{
}


SequentialSelector::SequentialSelector(Tree *tree, bool active)
{
	mTree = tree;
	mTreeItem = 0;
	mActive = active;
}

SequentialSelector::~SequentialSelector()
{

}

Item *SequentialSelector::next()
{
	if (mTreeItem)
	{
		mTreeItem = mTreeItem->nextPlayable();
	}
	else
	{
		mTreeItem = mTree->firstChild();
	}
	set();
	if (mTreeItem && mTreeItem->file())
		return new Item(mTreeItem->file());
	return 0;
}

Item *SequentialSelector::previous()
{
	TreeItem *back = mTree->firstChild();
	TreeItem *after;
	// now we just go forward on back until the item after back is me ;)
	// this is terribly unoptimized, but I'm terribly lazy
	while (back and (after = back->nextPlayable()) != mTreeItem)
	{
		back = after;
	}
	mTreeItem = back;
	set();
	if (mTreeItem && mTreeItem->file())
		return new Item(mTreeItem->file());
	return 0;
}

Item *SequentialSelector::current()
{
	if (!mTreeItem) return next();
	if (mTreeItem->file())
		return new Item(mTreeItem->file());
	return 0;
}

void SequentialSelector::setCurrent(const Item &item)
{
	mTreeItem = mTree->find(item.itemFile());
	if (mActive)
	{
		set();
		napp->player()->stop();
		napp->player()->play();
	}
}

void SequentialSelector::setAsCurrent(const Item &item)
{
	mTreeItem = mTree->find(item.itemFile());
}

void SequentialSelector::set()
{
	unless (mTreeItem) return;

	if (mActive)
		mTree->setCurrent(mTreeItem);
}



RandomSelector::RandomSelector(Tree *tree)
{
	mTree = tree;
	mPrevious = 0;
	mCurrent = 0;
}

Item *RandomSelector::next()
{
	File file;

	mPrevious = mCurrent;
	mCurrent=0;

	int tries=10;

	do
	{ // try it until the file actually exists
		int randomFileId =
			KApplication::random() % (mTree->oblique()->base()->high());
		randomFileId++;

		file = mTree->oblique()->base()->find(randomFileId);
	} while (!file && tries--);

	if (!file)
	{
		// prevent infinite loop
		// TODO: How would I cause this to efficiently always return an
		// item, even if the database has lots of holes?
		return 0;
	}

	// now I get to check if
	mCurrent = mTree->find(file);
	mTree->setCurrent(mCurrent);
	return new Item(file);
}

Item *RandomSelector::previous()
{
	if (!mPrevious) return 0;
	mCurrent = mPrevious;
	mPrevious = 0;

	mTree->setCurrent(mCurrent);
	return new Item(mCurrent->file());
}

Item *RandomSelector::current()
{
	return new Item(mCurrent->file());
}

void RandomSelector::setCurrent(const Item &item)
{
	mPrevious = 0;
	mCurrent = mTree->find(item.itemFile());
	mTree->setCurrent(mCurrent);

	napp->player()->stop();
	napp->player()->play();
}

void RandomSelector::setAsCurrent(const Item &item)
{
	mCurrent = mTree->find(item.itemFile());
}



